/* 
    berechnen der k-vektoren k=(k1*b1+k2*b2)
    der durch (L1,L2)=(l11*a1+l12*a2,l21*a1+l22*a2) aufgespannten periodischen
    Gitterausschnitts (a1,a2 sind Basisvektoren im direkten Raum,
    b1,b2 die reziproken Basisvektoren, (bi,aj)=2*\pi\delta(i,j) )
    
    da exp(i k*Li)=1 gelten muss, istfolgendes G-System zu loesen:
    
      (k1,k2)*((l11,l12),(l21,l22))=(n1,n2),  ki sind dann vielfache von vol(L)
    
    
    Bemerkung:
    - 2\pi wird manchmal aus bi rausgezogen und in exp(..) geschrieben

    compile: gcc -o kvec -lm kvec.c
    
*/
#include <stdlib.h>
#include <math.h>    /* sqrt() */

#define Version "0.2 Apr 1999"

int main(int argn,char *argv[]){
   int ll[2][2],i,j,vl;
   double a[2][2]={{1.0,0.0},{0.0,1.0}},b[2][2],ab,va,r1[2],r2[2];
   /*                     int_mcf_edgevectors double_basevectors */
   if(argn<5) 
   printf("program: kvec version "Version" JS\n"
          "  calculate the k-vektors for a finite lattice with PBC\n"
          "options: l11 l12 l21 l22 [a11 a12 a21 a22]\n"
          "  (l11,l12) (l21,l22) = mcf_edgevectors (integer)\n"
          "  (a11,a12) (a21,a22) = basevectors (double)\n");
   for(i=0;i<2;i++)
   for(j=0;j<2;j++) ll[i][j]=strtol(argv[i*2+j+1],NULL,10);
   if(argn>8)
   for(i=0;i<2;i++)
   for(j=0;j<2;j++)  a[i][j]=strtod(argv[i*2+j+5],NULL);
   vl=ll[0][0]*ll[1][1]-ll[0][1]*ll[1][0];
   va= a[0][0]* a[1][1]- a[0][1]* a[1][0];

   printf(" Va= %f  Vl= %d\n k in units of (b1,b2)\n",va,vl);
   for(i=-vl/2+1-vl%2;i<=vl/2;i++)		/* solve EqSystem */
   for(j=-vl/2+1-vl%2;j<=vl/2;j++)		/* -vl%2 Nov99 */
   if( (i*ll[0][0]+j*ll[0][1])%vl==0 
    && (i*ll[1][0]+j*ll[1][1])%vl==0 )
   printf(" k= % .4f % .4f = %3d/%d %3d/%d\n",i*1.0/vl,j*1.0/vl,i,vl,j,vl);
   
   /* besser brillionzone angeben */
   /* othogonalisati ai*bj=2\pi\delta(i,j) */
   for(i=0;i<2;i++)
   for(j=0;j<2;j++) /* invers=adj/det */
     b[i][j]=((i+j&1)?-1:1)*a[1-i][1-j]/va;
   printf(" a1    =(% f,% f)\n a2    =(% f,% f)\n"
          " b1/2pi=(% f,% f)\n b2/2pi=(% f,% f)\n"
          " k/2pi in direct units (quad BZ)\n",
     a[0][0],a[0][1],a[1][0],a[1][1],
     b[0][0],b[0][1],b[1][0],b[1][1]);
   for(i=-vl+1;i<=vl;i++)		/* solve EqSystem */
   for(j=-vl+1;j<=vl;j++)
   if( (i*ll[0][0]+j*ll[0][1])%vl==0 
    && (i*ll[1][0]+j*ll[1][1])%vl==0 )
   printf(" % .4f % .4f\n",i*b[0][0]/vl+j*b[1][0]/vl
                          ,i*b[0][1]/vl+j*b[1][1]/vl);
   /* ther are only two types of brillouin zones,
      rectangular or hexagonal */
   if( a[0][0]*a[1][0]+a[0][1]*a[1][1]==0 )
   {
     printf("# Brillouin-Zone: (xi,yi) in direct and base units\n");
     printf(" % .4f % .4f -  0.5  0.5\n",( b[0][0]+b[1][0])/2,( b[0][1]+b[1][1])/2);
     printf(" % .4f % .4f -  0.5 -0.5\n",( b[0][0]-b[1][0])/2,( b[0][1]-b[1][1])/2);
     printf(" % .4f % .4f - -0.5  0.5\n",(-b[0][0]+b[1][0])/2,(-b[0][1]+b[1][1])/2);
     printf(" % .4f % .4f - -0.5 -0.5\n",(-b[0][0]-b[1][0])/2,(-b[0][1]-b[1][1])/2);
   } else
   {
     printf("# Brillouin-Zone: (xi,yi) in direct and base units\n");
     j=1; if( b[0][0]*b[1][0]+b[0][1]*b[1][1] < 0 ) j=-1;
     for(i=0;i<6;i++)
     { /* 90deg rotation (y,-x) (Mittelsenkrechten) */
       if(i==0){
         r1[0]=( b[0][0]          )/2; r1[1]=( b[0][1]          )/2;
         r2[0]=( b[0][0]-j*b[1][0])/2; r2[1]=( b[0][1]-j*b[1][1])/2;
       }
       if(i==1){
         r1[0]=( b[0][0]-j*b[1][0])/2; r1[1]=( b[0][1]-j*b[1][1])/2;
         r2[0]=(        -j*b[1][0])/2; r2[1]=(        -j*b[1][1])/2;
       }
       if(i==2){
         r1[0]=(        -j*b[1][0])/2; r1[1]=(        -j*b[1][1])/2;
         r2[0]=(-b[0][0]          )/2; r2[1]=(-b[0][1]          )/2;
       }
       if(i==3){
         r1[0]=(-b[0][0]          )/2; r1[1]=(-b[0][1]          )/2;
         r2[0]=(-b[0][0]+j*b[1][0])/2; r2[1]=(-b[0][1]+j*b[1][1])/2;
       }
       if(i==4){
         r1[0]=(-b[0][0]+j*b[1][0])/2; r1[1]=(-b[0][1]+j*b[1][1])/2;
         r2[0]=(         j*b[1][0])/2; r2[1]=(         j*b[1][1])/2;
       }
       if(i==5){
         r1[0]=(         j*b[1][0])/2; r1[1]=(         j*b[1][1])/2;
         r2[0]=( b[0][0]          )/2; r2[1]=( b[0][1]          )/2;
       }
       ab=-(r2[0]*(r2[0]-r1[0])+r2[1]*(r2[1]-r1[1]))
         /(r1[0]*r2[1]-r1[1]*r2[0]);
//       printf(" r1=% .3f % .3f r2=% .3f % .3f x=% .3f",r1[0],r1[1],r2[0],r2[1],ab);
       r2[0]=r1[0]+ab*r1[1];  /* cross point */
       r2[1]=r1[1]-ab*r1[0];
       r1[0]=(r2[0]*b[1][1]-r2[1]*b[1][0])/(b[0][0]*b[1][1]-b[0][1]*b[1][0]); /* other koord */
       r1[1]=(r2[0]*b[0][1]-r2[1]*b[0][0])/(b[1][0]*b[0][1]-b[1][1]*b[0][0]);
       printf(" % .4f % .4f = % .4f % .4f\n",r2[0],r2[1],r1[0],r1[1]);

     }
   }
   return 0;
}
   